#!/bin/sh

PATH="/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin:$PATH"
MOUNT_ROOT="$1"
LOG="/dev/null"

OLD_IFS="$IFS"
IFS=':'
for PREFIX in $PATH; do
    [ "x${LVM_TOOL}" = "x" -a -x "$PREFIX/lvm" ] && LVM_TOOL="$PREFIX/lvm"
    [ "x${NTFS3G_TOOL}" = "x" -a -x "$PREFIX/ntfs-3g" ] && NTFS3G_TOOL="$PREFIX/ntfs-3g"
done
IFS="$OLD_IFS"

do_mount()
{
    #QNAP fix
    if [ -e /etc/config/qpkg.conf ]; then
        if [ "${3}" = "fat16" -o "${3}" = "fat32" ]; then
            FS_TYPE_ARG="-tvfat"
        elif [ "${3}" = "ntfs" ]; then
            FS_TYPE_ARG="-tufsd"
            NTFS3G_TOOL=''
        else
            FS_TYPE_ARG="-t${3}"
        fi
    fi

    #NETGEAR ReadyNAS fix
    if [ -d /proc/readynas ]; then
        if [ "${3}" = "ntfs" ]; then
            FS_TYPE_ARG="-tufsd"
        elif [ "${3}" = "fat32" ]; then
            FS_TYPE_ARG="-tvfat"
        else
            FS_TYPE_ARG="-t${3}"
        fi
    fi
    
    local CREATED=0
    if [ ! -d "$2" ]; then
        mkdir "$2" >>"$LOG" 2>&1
        CREATED=1
    fi

    #re-generate xfs & btrfs UUID
    if [ "${3}" = 'xfs' ]; then
        #xfs_admin -U generate "$1" >>"$LOG" 2>&1
        xfs_db -x -p bhsvc -c 'uuid generate' "$1" 2>&1 | grep ERROR >>"$LOG"
        if [ $? -eq 0 ]; then
            xfs_repair -L "$1" >>"$LOG" 2>&1
            xfs_db -x -p bhsvc -c 'uuid generate' "$1" >>"$LOG" 2>&1
        fi
    elif [ "${3}" = 'btrfs' ]; then
        #repair btrfs ref to original volume
        UUID=`blkid | sed -n '\!^'$1':.*UUID="\([^"]\+\)".*!s!!\1!p'`
        ORIGIN=`blkid | grep -v $1 | sed -n '\!^\([^:]\+\):.*UUID="'$UUID'".*!s!!\1!p' | head -n 1`
        if [ "x$ORIGIN" != "x" ]; then
            btrfs device scan "$ORIGIN" >>"$LOG" 2>&1 && \
            btrfstune -f -u "$1" >>"$LOG" 2>&1 && \
            (echo Y | btrfsck --repair "$1" >>"$LOG" 2>&1 )
            if [ $? -ne 0 ]; then
                return 100
            fi
        fi
    fi

    mount 2>/dev/null | grep -q "$1"
    if [ $? -eq 0 ]; then
        return 0
    fi
    
    local FSOPTIONS='ro,noauto,nodev,nosuid,noexec,noatime,nodiratime'
    if [ "${3}" = 'freebsd-ufs' ]; then
        FSOPTIONS="$FSOPTIONS,ufstype=ufs2"
    elif [ "${3}" = "fat16" -o "${3}" = "fat32" ]; then
        FSOPTIONS="$FSOPTIONS,utf8"
    fi

    if [ "${3}" = 'ntfs' -a "x${NTFS3G_TOOL}" != "x" ]; then
        "$NTFS3G_TOOL" -o "$FSOPTIONS" "$1" "$2" >>"$LOG" 2>&1
        local RESULT=$?
    else
        mount ${FS_TYPE_ARG} -o "$FSOPTIONS" "$1" "$2" >>"$LOG" 2>&1
        local RESULT=$?
    fi

    if [ $RESULT -ne 0 -a \( "${3}" = 'ext4' -o "${3}" = 'xfs' \) ]; then
        FSOPTIONS="$FSOPTIONS,norecovery"

        mount ${FS_TYPE_ARG} -o "$FSOPTIONS" "$1" "$2" >>"$LOG" 2>&1
        RESULT=$?
    fi

    if [ $RESULT -ne 0 -a $CREATED -eq 1 ]; then
        rm -rf "$2" >>"$LOG" 2>&1
    fi
    return $RESULT
}

TEST_PARTITIONS=""
TEST_PARTITIONS_FILTER=""

shift
for DEVICE in "$@"; do
    TYPE=`parted -s -m "$DEVICE" unit B print | sed -n '/.*\(msdos\|gpt\|loop\).*/s//\1/p'`
    PARTITIONS=`parted -s -m "$DEVICE" unit B print | sed -n '/^\([[:digit:]]\+.*\);$/s//\1/p' | grep -v msftres | cut -d : -f 1,4,5 | sed -e  's/B:/:/'`
    if [ "x${PARTITIONS}" = "x" ]; then
        PARTITIONS=`fdisk --bytes -l "$DEVICE" 2>/dev/null |  sed -n '\!^\(/dev/[a-z]\+\([0-9]\+\)\)[[:space:]*]\+[0-9]\+[[:space:]]\+[0-9]\+[[:space:]]\+[0-9]\+[[:space:]]\+\([0-9]\+\)[[:space:]+]\+\(82\|83\|b\|7\).*$!s!!\2:\3=\4!p' | sed -e 's/=82/:swap/' -e 's/=83/:ext4/' -e 's/=b/:fat32/' -e 's/=7/:ntfs/'`
        if [ "x${PARTITIONS}" = "x" ]; then
            PARTITIONS=`fdisk -l "$DEVICE" 2>/dev/null | sed -n '\!^\(/dev/.*\([0-9]\+\)\)[[:space:]*]\+[0-9]\+[[:space:]]\+[0-9]\+[[:space:]]\+\([0-9]\+\)[[:space:]+]\+\(82\|83\|b\|7\).*$!s!!\2:\3=\4!p' | sed -e 's/=82/:swap/' -e 's/=83/:ext4/' -e 's/=b/:fat32/' -e 's/=7/:ntfs/'`
            if [ "x${PARTITIONS}" = "x" ]; then
                TEST_PARTITIONS="$TEST_PARTITIONS $DEVICE"
                if [ "x$TEST_PARTITIONS_FILTER" = "x" ]; then
                    TEST_PARTITIONS_FILTER="(${DEVICE}:)"
                else
                    TEST_PARTITIONS_FILTER="${TEST_PARTITIONS_FILTER}|(${DEVICE}:)"
                fi
                continue
            fi
        fi
        TYPE="msdos"
    fi

    IFS="
"
    for PARTITION in ${PARTITIONS}; do
        INDEX=`echo $PARTITION | cut -d : -f 1`
        SIZE=`echo $PARTITION | cut -d : -f 2`
        FSTYPE=`echo $PARTITION | cut -d : -f 3`

        if [ "x${TYPE}" = "xloop" ]; then
            if [ ${INDEX:=1} -ne 1 ]; then
                break
            fi
            PARTITION_DEVICE="$DEVICE"
        else
            PARTITION_DEVICE="${DEVICE}${INDEX}"
            if [ ! -e "$PARTITION_DEVICE" ]; then
                PARTITION_DEVICE="${DEVICE}p${INDEX}"
            fi
        fi

        if [ "x${FSTYPE}" = "x" ]; then
            TEST_PARTITIONS="$TEST_PARTITIONS $PARTITION_DEVICE"
            if [ "x$TEST_PARTITIONS_FILTER" = "x" ]; then
                TEST_PARTITIONS_FILTER="(${PARTITION_DEVICE}:)"
            else
                TEST_PARTITIONS_FILTER="${TEST_PARTITIONS_FILTER}|(${PARTITION_DEVICE}:)"
            fi
            continue
        fi

        MOUNT=`basename "$PARTITION_DEVICE"`
        do_mount "$PARTITION_DEVICE" "$MOUNT_ROOT/$MOUNT" "$FSTYPE"
        if [ $? -ne 0 ]; then
            echo "$PARTITION_DEVICE $SIZE $FSTYPE"
        else
            echo "$PARTITION_DEVICE $SIZE $FSTYPE $MOUNT_ROOT/$MOUNT"
        fi
    done
done

if [ "x${TEST_PARTITIONS}" != "x" -a "x${LVM_TOOL}" != "x" ]; then
    VGS=`eval "$LVM_TOOL" pvs --noheadings --separator ':' -o pv_name,vg_name,lv_attr $TEST_PARTITIONS 2>/dev/null | grep -E "$TEST_PARTITIONS_FILTER" | sed -n '/[^:]\+:\([^:]\+\):....-.*$/s//\1/p' | sort -u`
    IFS="
"
    for VG in $VGS; do
        "$LVM_TOOL" vgchange -a ey "$VG" >>"$LOG" 2>&1
    done
fi

exit 0
